@@ -11,7 +11,9 @@ from pprint import pformat
 from . import stores
 from . import exceptions as exc
 from .utils import branches, getenv, parse_paths
-from .utils import log, logd
+from .utils import log, logd, sxpr_to_python, python_to_sxpr
+
+# TODO multi-get to reduce reads via context manager maybe?
 
 try:
     import yaml  # FIXME DANGERZONE :/
@@ -180,6 +182,7 @@ class ConfigBase:
             self._load_type = {None: self._load_runtime,
                                '.json': self._load_json,
                                '.py': self._load_python,
+                               '.sxpr': self._load_sxpr,
                                '.yaml': self._load_yaml}[key]
         except KeyError as e:
             raise exc.UnsupportedConfigLangError(path.suffix) from e
@@ -218,6 +221,17 @@ class ConfigBase:
         else:
             self._include = tuple()
 
+        # XXX there is a tradeoff here, if we defer loading, then we
+        # can start up faster, however, if there is an error in the
+        # config, then things become very confusing and hard to debug
+        # therefor on the balance we are better off detecting errors
+        # in configs at load time so that it is clear what went wrong
+        # we might be able to improve this a tiny bit by keeping a
+        # single file that records paths and modified times for
+        # configs so that we only have to read a single file and stat
+        # the rest which should be faster than what we do right now,
+        # however that is a micro optimization at this point, however
+        # it may be worth it as the number of configs in use increases
         if self._path is not None:
             self.load()
 
@@ -264,7 +278,8 @@ class ConfigBase:
             except json.decoder.JSONDecodeError as e:
                 raise empty_error from e
             except SyntaxError as e:
-                if e.msg == 'unexpected EOF while parsing' and e.offset == 0:
+                if (e.msg in ('unexpected EOF while parsing', 'invalid syntax') and
+                    e.offset == 0):
                     raise empty_error from e
                 else:
                     raise e
@@ -288,6 +303,9 @@ class ConfigBase:
             return json.dumps(config, indent=2, sort_keys=True)
         elif format == 'py':
             return pformat(config)
+        elif format == 'sxpr':
+            # XXX expect failure for any complex types
+            return python_to_sxpr(config)
         elif format == 'yaml':
             try:
                 return yaml.dump(config, default_flow_style=False)
@@ -316,6 +334,7 @@ class ConfigBase:
     def _load_string(self, string, format):
         loadf = {'json': json.loads,
                  'py': ast.literal_eval,
+                 'sxpr': sxpr_to_python,
                  'yaml': yaml.safe_load,}[format]
         return loadf(string)
 
@@ -345,6 +364,10 @@ class ConfigBase:
         with open(self._path, 'rt') as f:
             return ast.literal_eval(f.read())
 
+    def _load_sxpr(self):
+        with open(self._path, 'rt') as f:
+            return sxpr_to_python(f.read())
+
     def _load_yaml(self):
         with open(self._path, 'rt') as f:
             try:
@@ -360,14 +383,20 @@ class ConfigBase:
         envars = []
         # env
         for evkey in ('environment-variables', 'env-vars', 'envars'):
+            # FIXME do we allow multiple keys like this or error if
+            # more than one is detected?
             if evkey in auth_variable_value:
-                ev = auth_variable_value[evkey]
-                if isinstance(ev, str):
-                    envars += ev.split(' ')
-                elif isinstance(ev, list):
-                    envars += ev
-                else:
-                    raise TypeError(f'unsupported type {type(ev)}\n{ev}')
+                if evkey in auth_variable_value:
+                    ev = auth_variable_value[evkey]
+                    if isinstance(ev, str):
+                        envars += ev.split(' ')
+                    elif isinstance(ev, list):
+                        envars += ev
+                    else:
+                        # this is actually a type error because it means
+                        # something is wrong with the implementation not
+                        # with what the user did
+                        raise TypeError(f'unsupported type {type(ev)}\n{ev}')
 
         return envars
 
@@ -375,7 +404,8 @@ class ConfigBase:
     def _auth_paths(auth_variable_value):
         if 'config' in auth_variable_value:
             if [_ for _ in ('path', 'paths', 'paths-nested') if _ in auth_variable_value]:
-                raise TypeError('can only have config or path, not both')
+                msg = ('can only have config or path, not both')
+                raise exc.BadAuthConfigFormatError(msg)
 
             return []
 
@@ -543,6 +573,7 @@ class AuthConfig(DecoBase, ConfigBase):  # FIXME this is more a schema?
 
     def load(self):
         if not hasattr(self, '_blob'):
+            # auth configs are static so it is ok to cache them
             self._blob = super().load()
 
         return self._blob
@@ -685,7 +716,7 @@ class AuthConfig(DecoBase, ConfigBase):  # FIXME this is more a schema?
             for i in self._include:
                 try:
                     return i.get(variable_name, **kwargs)
-                except KeyError as e:
+                except KeyError as e:  # is indeed a key error
                     error = e
             else:
                 if error is not None:
@@ -693,9 +724,13 @@ class AuthConfig(DecoBase, ConfigBase):  # FIXME this is more a schema?
 
         try:
             user_variable_value = self.user_config.get_blob('auth-variables', variable_name)
+            # we use get_blob to reduce the number of times we read from disk, we dereference
+            # the value in secrets below if there is one, but we defer that until later, so
+            # do not call self.user_config.get() at this point
             if user_variable_value is None:
                 user_variable_value = {}
-                f1 = True
+                f1 = TypeError(f'Value of {variable_name} is None'
+                               f' in user config {self.user_config._path}')
             else:
                 f1 = False
         except KeyError as e:
@@ -796,7 +831,7 @@ class AuthConfig(DecoBase, ConfigBase):  # FIXME this is more a schema?
 
         envars = self._envars(user_variable_value)
         envars += self._envars(auth_variable_value)
-        paths = self._auth_paths(user_variable_value)
+        paths = self._auth_paths(user_variable_value)  # this is not expected to raise a KeyError
         paths += self._auth_paths(auth_variable_value)
 
         bads = self._single_alt_configs(auth_variable_value)  # for error purposes only
@@ -942,6 +977,7 @@ class UserConfig(ConfigBase):
 
     def get(self, variable_name, *args, **kwargs):
         """ look up the value of a variable name from auth store or config """
+        # XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX this is NOT CALLED by AuthConfig.get !!!!!!!!!!!!!!!
         # ah the problem of interleveing values from sources of different rank ...
         for_list = 'for_list' in kwargs and kwargs['for_list']
         for_path = 'for_path' in kwargs and kwargs['for_path']
@@ -1003,17 +1039,17 @@ class UserConfig(ConfigBase):
             auth_store = self.path_source(*names)  # FIXME perf issue incoming
             try:
                 return auth_store._path, auth_store(*names)
-            except KeyError as e:
+            except exc.SecretError as e:
                 errors.append(e)
                 logd.error(f'broken path {names}')
                 if auth_store != secrets:
                     try:
                         return secrets._path, secrets(*names)
-                    except KeyError as e:
+                    except exc.SecretError as e:
                         errors.append(e)
 
         if errors:
-            raise KeyError(f'{[e.args[0] for e in errors]}') from errors[-1]
+            raise exc.SecretError(f'{[e.args[0] for e in errors]}') from errors[-1]
 
     def _gsac_wrap(self, args):
         return self._get_single_alt_config(*args)
@@ -1057,8 +1093,9 @@ class UserConfig(ConfigBase):
             _test.pop('alt-config')
             rename = _test.pop('rename', None)
             if _test:
-                raise ValueError('configs with top level alt-config may '
-                                f'have only a rename section\n{_test}')
+                msg = ('configs with top level alt-config may '
+                       f'have only a rename section\n{_test}')
+                raise exc.BadAuthConfigFormatError(msg)
 
             self._c_alt_config = self._from_user_alt_config(path,
                                                             self.auth_config,
